@@ -3,13 +3,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <getopt.h>
 #include <errno.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <sysexits.h>
 #include <sys/ioctl.h>
 #include <linux/kd.h>
-#include "kdmapop.h"
+
+#include <kfont.h>
 
 #include "libcommon.h"
 
@@ -27,10 +29,14 @@ ud_compar(const void *u1, const void *u2)
 	return (int)fp1 - (int)fp2;
 }
 
-static void __attribute__((noreturn))
-usage(int rc)
+static void KBD_ATTR_NORETURN
+usage(int rc, const struct kbd_help *options)
 {
-	fprintf(stderr, _("Usage:\n\t%s [-s] [-C console]\n"), get_progname());
+	fprintf(stderr, _("Usage: %s [option...]\n"), program_invocation_short_name);
+
+	print_options(options);
+	print_report_bugs();
+
 	exit(rc);
 }
 
@@ -43,14 +49,25 @@ int main(int argc, char **argv)
 	char *console = NULL;
 	struct unimapdesc ud;
 
-	set_progname(argv[0]);
 	setuplocale();
 
-	if (argc == 2 &&
-	    (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version")))
-		print_version_and_exit();
+	const char *const short_opts = "hVsC:";
+	const struct option long_opts[] = {
+		{ "sort",    no_argument,       NULL, 's' },
+		{ "console", required_argument, NULL, 'C' },
+		{ "help",    no_argument,       NULL, 'h' },
+		{ "version", no_argument,       NULL, 'V' },
+		{ NULL, 0, NULL, 0 }
+	};
+	const struct kbd_help opthelp[] = {
+		{ "-s, --sort",        _("sort and merge elements.") },
+		{ "-C, --console=DEV", _("the console device to be used.") },
+		{ "-V, --version",     _("print version number.")     },
+		{ "-h, --help",        _("print this usage message.") },
+		{ NULL, NULL }
+	};
 
-	while ((c = getopt(argc, argv, "sC:")) != EOF) {
+	while ((c = getopt_long(argc, argv, short_opts, long_opts, NULL)) != -1) {
 		switch (c) {
 			case 's':
 				sortflag = 1;
@@ -58,18 +75,31 @@ int main(int argc, char **argv)
 			case 'C':
 				console = optarg;
 				break;
-			default:
-				usage(EX_USAGE);
+			case 'V':
+				print_version_and_exit();
+				break;
+			case 'h':
+				usage(EXIT_SUCCESS, opthelp);
+				break;
+			case '?':
+				usage(EX_USAGE, opthelp);
+				break;
 		}
 	}
 
 	if (optind < argc)
-		usage(EX_USAGE);
+		usage(EX_USAGE, opthelp);
 
 	if ((fd = getfd(console)) < 0)
-		kbd_error(EXIT_FAILURE, 0, _("Couldn't get a file descriptor referring to the console"));
+		kbd_error(EXIT_FAILURE, 0, _("Couldn't get a file descriptor referring to the console."));
 
-	if (getunimap(fd, &ud))
+	int ret;
+	struct kfont_context *kfont;
+
+	if ((ret = kfont_init(program_invocation_short_name, &kfont)) < 0)
+		return -ret;
+
+	if (kfont_get_unicodemap(kfont, fd, &ud))
 		return EXIT_FAILURE;
 
 	if (sortflag) {
